@@ -32,9 +32,9 @@ static char **filelist = NULL;
 		/* The list of files to display in the file browser. */
 static size_t filelist_len = 0;
 		/* The number of files in the list. */
-static int width = 0;
+static size_t width = 0;
 		/* The number of files that we can display per screen row. */
-static int longest = 0;
+static size_t longest = 0;
 		/* The number of columns in the longest filename in the list. */
 static size_t selected = 0;
 		/* The currently selected filename in the list; zero-based. */
@@ -187,10 +187,10 @@ char *do_browser(char *path)
 			selected += width - 1 - (selected % width);
 			if (selected >= filelist_len)
 				selected = filelist_len - 1;
-		} else if (func == do_up_void) {
+		} else if (func == do_up) {
 			if (selected >= width)
 				selected -= width;
-		} else if (func == do_down_void) {
+		} else if (func == do_down) {
 			if (selected + width <= filelist_len - 1)
 				selected += width;
 		} else if (func == do_prev_block) {
@@ -258,9 +258,9 @@ char *do_browser(char *path)
 
 			/* In case the specified directory cannot be entered, select it
 			 * (if it is in the current list) so it will be highlighted. */
-			for (i = 0; i < filelist_len; i++)
-				if (strcmp(filelist[i], path) == 0)
-					selected = i;
+			for (size_t j = 0; j < filelist_len; j++)
+				if (strcmp(filelist[j], path) == 0)
+					selected = j;
 
 			/* Try opening and reading the specified directory. */
 			goto read_directory_contents;
@@ -303,6 +303,10 @@ char *do_browser(char *path)
 			/* Try opening and reading the selected directory. */
 			path = mallocstrcpy(path, filelist[selected]);
 			goto read_directory_contents;
+#ifdef ENABLE_NANORC
+		} else if (func == (functionptrtype)implant) {
+			implant(first_sc_for(MBROWSER, func)->expansion);
+#endif
 		} else if (func == do_exit) {
 			/* Exit from the file browser. */
 			break;
@@ -357,7 +361,7 @@ char *do_browse_from(const char *inpath)
 		path = free_and_assign(path, strip_last_component(path));
 
 		if (stat(path, &st) == -1 || !S_ISDIR(st.st_mode)) {
-			char * currentdir = charalloc(PATH_MAX + 1);
+			char *currentdir = charalloc(PATH_MAX + 1);
 
 			free(path);
 			path = getcwd(currentdir, PATH_MAX + 1);
@@ -521,7 +525,7 @@ void browser_refresh(void)
 				/* The length of the filename in columns. */
 		size_t infolen;
 				/* The length of the file information in columns. */
-		int infomaxlen = 7;
+		size_t infomaxlen = 7;
 				/* The maximum length of the file information in columns:
 				 * normally seven, but will be twelve for "(parent dir)". */
 		bool dots = (COLS >= 15 && namelen >= longest - infomaxlen);
