@@ -89,113 +89,106 @@ void search_replace_abort(void)
 	regexp_cleanup();
 }
 
-/* Set up the system variables for a search or replace.  If use_answer
- * is TRUE, only set backupstring to answer.  Return -2 to run the
- * opposite program (search -> replace, replace -> search), return -1 if
- * the search should be canceled (due to Cancel, a blank search string,
- * Go to Line, or a failed regcomp()), return 0 on success, and return 1
- * on rerun calling program.
- *
- * replacing is TRUE if we call from do_replace(), and FALSE if called
- * from do_search(). */
-int search_init(bool replacing, bool use_answer)
+/* Prepare the prompt and ask the user what to search for.  Keep looping
+ * as long as the user presses a toggle, and only take action and exit
+ * when <Enter> is pressed or a non-toggle shortcut was executed. */
+void search_init(bool replacing, bool keep_the_answer)
 {
-	int i = 0;
-	char *buf;
-	static char *backupstring = NULL;
-		/* The search string we'll be using. */
-	functionptrtype func;
+	char *thedefault;
+		/* What will be searched for when the user typed nothing. */
 
-	/* If use_answer is TRUE, set backupstring to answer and get out. */
-	if (use_answer) {
-		backupstring = mallocstrcpy(backupstring, answer);
-		return 0;
-	}
-
-	/* We display the search prompt below.  If the user types a partial
-	 * search string and then Replace or a toggle, we will return to
-	 * do_search() or do_replace() and be called again.  In that case,
-	 * we should put the same search string back up. */
+	/* When starting a new search, clear the current answer. */
+	if (!keep_the_answer)
+		answer = mallocstrcpy(answer, NULL);
 
+	/* If something was searched for earlier, include it in the prompt. */
 	if (*last_search != '\0') {
 		char *disp = display_string(last_search, 0, COLS / 3, FALSE);
 
-		buf = charalloc(strlen(disp) + 7);
+		thedefault = charalloc(strlen(disp) + 7);
 		/* We use (COLS / 3) here because we need to see more on the line. */
-		sprintf(buf, " [%s%s]", disp,
+		sprintf(thedefault, " [%s%s]", disp,
 				(strlenpt(last_search) > COLS / 3) ? "..." : "");
 		free(disp);
 	} else
-		buf = mallocstrcpy(NULL, "");
+		thedefault = mallocstrcpy(NULL, "");
 
-	/* This is now one simple call.  It just does a lot. */
-	i = do_prompt(FALSE, FALSE,
-				inhelp ? MFINDINHELP : (replacing ? MREPLACE : MWHEREIS),
-				backupstring, &search_history,
-				/* TRANSLATORS: This is the main search prompt. */
-				edit_refresh, "%s%s%s%s%s%s", _("Search"),
-				/* TRANSLATORS: The next three modify the search prompt. */
-				ISSET(CASE_SENSITIVE) ? _(" [Case Sensitive]") : "",
-				ISSET(USE_REGEXP) ? _(" [Regexp]") : "",
-				ISSET(BACKWARDS_SEARCH) ? _(" [Backwards]") : "", replacing ?
+	while (TRUE) {
+		functionptrtype func;
+		/* Ask the user what to search for (or replace). */
+		int i = do_prompt(FALSE, FALSE,
+					inhelp ? MFINDINHELP : (replacing ? MREPLACE : MWHEREIS),
+					answer, &search_history,
+					/* TRANSLATORS: This is the main search prompt. */
+					edit_refresh, "%s%s%s%s%s%s", _("Search"),
+					/* TRANSLATORS: The next three modify the search prompt. */
+					ISSET(CASE_SENSITIVE) ? _(" [Case Sensitive]") : "",
+					ISSET(USE_REGEXP) ? _(" [Regexp]") : "",
+					ISSET(BACKWARDS_SEARCH) ? _(" [Backwards]") : "", replacing ?
 #ifndef NANO_TINY
-				/* TRANSLATORS: The next two modify the search prompt. */
-				openfile->mark ? _(" (to replace) in selection") :
+					/* TRANSLATORS: The next two modify the search prompt. */
+					openfile->mark ? _(" (to replace) in selection") :
 #endif
-				_(" (to replace)") : "", buf);
+					_(" (to replace)") : "", thedefault);
 
-	/* Release buf now that we don't need it anymore. */
-	free(buf);
+		/* If the search was cancelled, or we have a blank answer and
+		 * nothing was searched for yet during this session, get out. */
+		if (i == -1 || (i == -2 && *last_search == '\0')) {
+			statusbar(_("Cancelled"));
+			search_replace_abort();
+			free(thedefault);
+			return;
+		}
 
-	free(backupstring);
-	backupstring = NULL;
+		/* If Enter was pressed, prepare to do a replace or a search. */
+		if (i == 0 || i == -2) {
+			/* If an actual answer was typed, remember it. */
+			if (*answer != '\0') {
+				last_search = mallocstrcpy(last_search, answer);
+#ifdef ENABLE_HISTORIES
+				update_history(&search_history, answer);
+#endif
+			}
+
+			free(thedefault);
+
+			/* If doing a regular-expression search, compile the
+			 * search string, and get out when it's invalid. */
+			if (ISSET(USE_REGEXP) && !regexp_init(last_search)) {
+				search_replace_abort();
+				return;
+			}
 
-	/* If the search was cancelled, or we have a blank answer and
-	 * nothing was searched for yet during this session, get out. */
-	if (i == -1 || (i == -2 && *last_search == '\0')) {
-		statusbar(_("Cancelled"));
-		return -1;
-	}
+			if (replacing)
+				ask_for_replacement();
+			else
+				go_looking();
 
-	/* If Enter was pressed, see what we got. */
-	if (i == 0 || i == -2) {
-		/* If an answer was given, remember it. */
-		if (*answer != '\0') {
-			last_search = mallocstrcpy(last_search, answer);
-#ifdef ENABLE_HISTORIES
-			update_history(&search_history, answer);
-#endif
+			search_replace_abort();
+			return;
+		}
+
+		func = func_from_key(&i);
+
+		/* If we're here, one of the five toggles was pressed, or
+		 * a shortcut was executed. */
+		if (func == case_sens_void) {
+			TOGGLE(CASE_SENSITIVE);
+		} else if (func == backwards_void) {
+			TOGGLE(BACKWARDS_SEARCH);
+		} else if (func == regexp_void) {
+			TOGGLE(USE_REGEXP);
+		} else if (func == flip_replace) {
+			replacing = !replacing;
+		} else {
+			if (func == flip_goto)
+				do_gotolinecolumn(openfile->current->lineno,
+							openfile->placewewant + 1, TRUE, TRUE);
+			search_replace_abort();
+			free(thedefault);
+			return;
 		}
-		if (ISSET(USE_REGEXP) && !regexp_init(last_search))
-			return -1;
-		else
-			return 0;    /* We have a valid string or regex. */
 	}
-
-	func = func_from_key(&i);
-
-	if (func == case_sens_void) {
-		TOGGLE(CASE_SENSITIVE);
-		backupstring = mallocstrcpy(backupstring, answer);
-		return 1;
-	} else if (func == backwards_void) {
-		TOGGLE(BACKWARDS_SEARCH);
-		backupstring = mallocstrcpy(backupstring, answer);
-		return 1;
-	} else if (func == regexp_void) {
-		TOGGLE(USE_REGEXP);
-		backupstring = mallocstrcpy(backupstring, answer);
-		return 1;
-	} else if (func == flip_replace) {
-		backupstring = mallocstrcpy(backupstring, answer);
-		return -2;    /* Call the opposite search function. */
-	} else if (func == do_gotolinecolumn_void) {
-		do_gotolinecolumn(openfile->current->lineno,
-						openfile->placewewant + 1, TRUE, TRUE);
-		return 3;
-	}
-
-	return -1;
 }
 
 /* Look for needle, starting at (current, current_x).  begin is the line
@@ -353,17 +346,7 @@ int findnextstr(const char *needle, bool whole_word_only, int modus,
 /* Ask what to search for and then go looking for it. */
 void do_search(void)
 {
-	int i = search_init(FALSE, FALSE);
-
-	if (i == -1)    /* Cancelled, or some other exit reason. */
-		search_replace_abort();
-	else if (i == -2)    /* Do a replace instead. */
-		do_replace();
-	else if (i == 1)    /* Toggled something. */
-		do_search();
-
-	if (i == 0)
-		go_looking();
+	search_init(FALSE, FALSE);
 }
 
 /* Search forward for a string. */
@@ -418,6 +401,8 @@ void do_research(void)
 	currmenu = MWHEREIS;
 
 	go_looking();
+
+	search_replace_abort();
 }
 
 /* Search for the global string 'last_search'.  Inform the user when
@@ -448,7 +433,6 @@ void go_looking(void)
 #endif
 
 	edit_redraw(was_current, CENTERING);
-	search_replace_abort();
 }
 
 /* Calculate the size of the replacement text, taking possible
@@ -715,30 +699,20 @@ ssize_t do_replace_loop(const char *needle, bool whole_word_only,
 
 /* Replace a string. */
 void do_replace(void)
+{
+	if (ISSET(VIEW_MODE))
+		print_view_warning();
+	else
+		search_init(TRUE, FALSE);
+}
+
+/* Ask the user what the already given search string should be replaced with. */
+void ask_for_replacement(void)
 {
 	filestruct *edittop_save, *begin;
 	size_t firstcolumn_save, begin_x;
 	ssize_t numreplaced;
-	int i;
-
-	if (ISSET(VIEW_MODE)) {
-		print_view_warning();
-		return;
-	}
-
-	i = search_init(TRUE, FALSE);
-
-	if (i == -1)    /* Cancelled, or some other exit reason. */
-		search_replace_abort();
-	else if (i == -2)    /* Do a search instead. */
-		do_search();
-	else if (i == 1)    /* Toggled something. */
-		do_replace();
-
-	if (i != 0)
-		return;
-
-	i = do_prompt(FALSE, FALSE, MREPLACEWITH, NULL, &replace_history,
+	int i = do_prompt(FALSE, FALSE, MREPLACEWITH, NULL, &replace_history,
 				/* TRANSLATORS: This is a prompt. */
 				edit_refresh, _("Replace with"));
 
@@ -752,7 +726,6 @@ void do_replace(void)
 	if (i == -1 || i > 0) {
 		if (i == -1)
 			statusbar(_("Cancelled"));
-		search_replace_abort();
 		return;
 	}
 
@@ -774,8 +747,6 @@ void do_replace(void)
 	if (numreplaced >= 0)
 		statusline(HUSH, P_("Replaced %zd occurrence",
 				"Replaced %zd occurrences", numreplaced), numreplaced);
-
-	search_replace_abort();
 }
 
 /* Go to the specified line and x position. */
@@ -798,8 +769,6 @@ void do_gotolinecolumn(ssize_t line, ssize_t column, bool use_answer,
 		bool interactive)
 {
 	if (interactive) {
-		functionptrtype func;
-
 		/* Ask for the line and column. */
 		int i = do_prompt(FALSE, FALSE, MGOTOLINE,
 				use_answer ? answer : NULL, NULL,
@@ -812,12 +781,10 @@ void do_gotolinecolumn(ssize_t line, ssize_t column, bool use_answer,
 			return;
 		}
 
-		func = func_from_key(&i);
-
-		if (func == gototext_void) {
+		if (func_from_key(&i) == flip_goto) {
 			/* Retain what the user typed so far and switch to searching. */
-			search_init(TRUE, TRUE);
-			do_search();
+			search_init(FALSE, TRUE);
+			return;
 		}
 
 		/* If a function was executed, we're done here. */
@@ -873,11 +840,11 @@ void do_gotolinecolumn(ssize_t line, ssize_t column, bool use_answer,
 
 #ifndef NANO_TINY
 		if (ISSET(SOFTWRAP)) {
-			filestruct *line = openfile->current;
+			filestruct *currentline = openfile->current;
 			size_t leftedge = leftedge_for(xplustabs(), openfile->current);
 
-			rows_from_tail = (editwinrows / 2) -
-						go_forward_chunks(editwinrows / 2, &line, &leftedge);
+			rows_from_tail = (editwinrows / 2) - go_forward_chunks(
+								editwinrows / 2, &currentline, &leftedge);
 		} else
 #endif
 			rows_from_tail = openfile->filebot->lineno -
